# Sample Problem 3: Inequality State Path Constraint
# (Ex 4 from Dynopt Guide)
#
#   min x3(tf)
#   s.t.    X1_dot = X2                     X1(0) =  0
#           X2_dot = -X2+u                  X2(0) = -1
#           X3_dot = X1^2+x2^2+0.005*u^2    X3(0) =  0
#           X2-8*(t-0.5)^2+0.5 <= 0
#           tf = 1
#

import coopr.environ
from coopr.pyomo import *
from coopr.dae import *

m = ConcreteModel()

m.t = DifferentialSet(bounds=(0,1))
m.x1 = Var(m.t)
m.x2 = Var(m.t)
m.x3 = Var(m.t)
m.u = Var(m.t, initialize=0)

m.obj = Objective(expr=m.x3[1])

def _init(m):
    yield m.x1[0] == 0
    yield m.x2[0] == -1
    yield m.x3[0] == 0
    yield ConstraintList.End
m.init_conditions = ConstraintList(rule=_init)

def _x1dot(m, t):
    return m.x2[t]
m.x1dot = Differential(dv=m.x1, rule=_x1dot)

def _x2dot(m, t):
    return -m.x2[t]+m.u[t]
m.x2dot = Differential(dv=m.x2, rule=_x2dot)

def _x3dot(m, t):
    return m.x1[t]**2+m.x2[t]**2+0.005*m.u[t]**2
m.x3dot = Differential(dv=m.x3, rule=_x3dot)

def _con(m, t):
    return m.x2[t]-8*(t-0.5)**2+0.5 <= 0
m.con = Constraint(m.t, rule=_con)

